<?php

namespace Acts\CamdramBundle\Entity;

use Doctrine\ORM\EntityRepository;

/**
 * PerformanceRepository
 *
 * This class was generated by the Doctrine ORM. Add your own custom
 * repository methods below.
 */
class PerformanceRepository extends EntityRepository
{
    /**
     * findInDateRange
     *
     * Find all authorized performances between two dates, joined to the
     * corresponding show.
     *
     * @param \DateTime $start
     * @param \DateTime $end
     *
     * @return array
     */
    public function findInDateRange(\DateTime $start, \DateTime $end)
    {
        $qb = $this->createQueryBuilder('p')
            ->join('p.show', 's')
            ->leftjoin('p.venue', 'v')
            ->addSelect('s')
            ->addSelect('v')
            ->where('s.authorised = true')
            ->andWhere('p.start_at < :end')
            ->andWhere('p.repeat_until >= :start')
            ->orderBy('p.start_at', 'ASC')
            ->setParameter('start', $start)
            ->setParameter('end', $end)
        ;

        return $qb->getQuery()->getResult();
    }

    public function getNumberInDateRange(\DateTime $start, \DateTime $end)
    {
        $qb = $this->createQueryBuilder('p');
        $qb->join('p.show', 's')
            ->where('p.start_at < :end')
            ->andWhere('p.repeat_until >= :start')
            ->andWhere('s.authorised = true')
            ->setParameter('start', $start)
            ->setParameter('end', $end);

        $result = $qb->getQuery()->getResult();
        $count = 0;
        $max_end = clone $end;
        $max_end->modify('-1 day');
        foreach ($result as $p) {
            if ($p->getShow()) {
                $start_at = $p->getStartAt();
                $end_at = $p->getRepeatUntil();
                if ($start_at < $start) {
                    $start_at = $start;
                }
                if ($end_at > $end) {
                    $end_at = $max_end;
                }

                $count += $end_at->diff($start_at)->d + 1;
            }
        }

        return $count;
    }

    public function getNumberOfVenueNamesInDateRange(\DateTime $start, \DateTime $end)
    {
        $qb = $this->createQueryBuilder('p')->select('COUNT(DISTINCT p.other_venue)')
            ->innerJoin('p.show', 's')
            ->where('s.authorised = true')
            ->andwhere('p.venue IS NULL')
            ->andWhere('p.start_at < :end')
            ->andWhere('p.repeat_until >= :start')
            ->setParameter('start', $start)
            ->setParameter('end', $end);
        $result = $qb->getQuery()->getOneOrNullResult();

        return current($result);
    }

    public function getBySociety(Society $society, \DateTime $from = null, \DateTime $to = null)
    {
        $query = $this->createQueryBuilder('p')
            ->join('p.show', 's')
            ->leftjoin('p.venue', 'v')
            ->addSelect('s')
            ->addSelect('v')
            ->where('s.authorised = true')
            ->andWhere(':society MEMBER OF s.societies');

        if ($from) {
            $query = $query->andWhere('p.start_at > :from')->setParameter('from', $from);
        }

        if ($to) {
            $query = $query->andWhere('p.repeat_until <= :to')->setParameter('to', $to);
        }

        $query = $query->orderBy('p.start_at', 'ASC')
            ->setParameter('society', $society)
            ->getQuery();

        return $query->getResult();
    }

    public function getByVenue(Venue $venue, \DateTime $from = null, \DateTime $to = null)
    {
        $query = $this->createQueryBuilder('p')
            ->join('p.show', 's')
            ->join('p.venue', 'v')
            ->addSelect('s')
            ->addSelect('v')
            ->where('s.authorised = true');

        if ($from) {
            $query = $query->andWhere('p.start_at > :from')->setParameter('from', $from);
        }

        if ($to) {
            $query = $query->andWhere('p.repeat_until <= :to')->setParameter('to', $to);
        }

        $query = $query->andWhere('p.venue = :venue')
            ->orderBy('p.start_at', 'ASC')
            ->setParameter('venue', $venue)
            ->getQuery();

        return $query->getResult();
    }

    public function getFirstDate()
    {
        $query = $this->createQueryBuilder('p')
            ->select('MIN(p.start_at)')
            ->join('p.show', 's')
            ->where('s.authorised = true')
            ->andWhere('p.start_at IS NOT NULL')
            ->andWhere('p.start_at > :min')
            ->setMaxResults(1)
            ->setParameter('min', new \DateTime('1990-01-01'))
            ->getQuery();

        return new \DateTime(current($query->getOneOrNullResult()));
    }

    public function getLastDate()
    {
        $query = $this->createQueryBuilder('p')
            ->select('MAX(p.repeat_until)')
            ->join('p.show', 's')
            ->where('s.authorised = true')
            ->setMaxResults(1)
            ->getQuery();

        return new \DateTime(current($query->getOneOrNullResult()));
    }
}
